{*******************************************************}
{                                                       }
{       xTool - Component Collection                    }
{                                                       }
{       Copyright (c) 1995 Stefan Bther                }
{                            stefc@fabula.com           }
{*******************************************************}

unit xProcs;


{$D-}
{$define SET}

interface

uses
 {$IFDEF Win32} Windows, Registry, {$ELSE} WinTypes, WinProcs, {$ENDIF}
  ShellAPI, Messages, Classes, Graphics;

const
  XCOMPANY        = 'Fabula Software';

const
  { several important ASCII codes }
  BACKSPACE       =  #8;
  TAB             =  #9;
  LF              = #10;
  CR              = #13;
  EOF_            = #26;    { 30.07.96 sb }
  ESC             = #27;
  BLANK           = #32;
  SPACE           = BLANK;

  { digits as chars }
  ZERO   = '0';  ONE  = '1';  TWO    = '2';  THREE  = '3';  FOUR  = '4';
  FIVE   = '5';  SIX  = '6';  SEVEN  = '7';  EIGHT  = '8';  NINE  = '9';

  { special codes }
  SLASH           = '\';     { used in filenames }
  HEX_PREFIX      = '$';     { prefix for hexnumbers }

  CRLF            : PChar = CR+LF;

  { Low Extendeding point value }
  FLTZERO         : Extended = 0.00000001;


{$ifdef SET}
  DIGITS          : set of Char = [ZERO..NINE];
{$endif}

  { important registry keys / items }
  REG_CURRENT_VERSION = 'Software\Microsoft\Windows\CurrentVersion';
  REG_CURRENT_USER    = 'RegisteredOwner';
  REG_CURRENT_COMPANY = 'RegisteredOrganization';

  PRIME_16       = 65521;
  PRIME_32       = 2147483647;

  MINSHORTINT    = -128;               { 1.8.96 sb }
  MAXSHORTINT    =  127;
  MINBYTE        =  0;
  MAXBYTE        =  255;
  MINWORD        =  0;
  MAXWORD        =  65535;


{$ifdef SET}
type
  TMonth        = (NoneMonth,January,February,March,April,May,June,July,
                   August,September,October,November,December);

  TDayOfWeek    = (Sunday,Monday,Tuesday,Wednesday,Thursday,Friday,Saturday);

  TBit          = 0..31;

  { Search and Replace options }
  TSROption     = (srWord,srCase,srAll);
  TSROptions    = set of TsrOption;

  { Data types }
  TDataType     = (dtInteger,dtBoolean,dtString,dtDate,dtTime,
                   dtExtended,dtCurrency);
{$endif}

var
  IsWin95,
  IsWinNT   : Boolean;
  xLanguage : Integer;
  xLangOfs  : Integer;

{ bit manipulating }

type

XProc = class
{$ifdef SET}
function bitSet(const Value: LongInt; const TheBit: TBit): Boolean;
function bitOn(const Value: LongInt; const TheBit: TBit): LongInt;
function bitOff(const Value: LongInt; const TheBit: TBit): LongInt;
function bitToggle(const Value: LongInt; const TheBit: TBit): LongInt;
{$endif}

{ String functions }
function  strHash(const S: String; LastBucket: Integer): Integer;
function  strCut(const S: String; Len: Integer): String;
function  strTrim(const S: String): String;
function  strTrimA(const S: String): String;
function  strTrimChA(const S: String; C: Char): String;
function  strTrimChL(const S: String; C: Char): String;
function  strTrimChR(const S: String; C: Char): String;
function  strLeft(const S: String; Len: Integer): String;
function  strLower(const S: String): String;
function  strMake(C: Char; Len: Integer): String;
function  strPadChL(const S: String; C: Char; Len: Integer): String;
function  strPadChR(const S: String; C: Char; Len: Integer): String;
function  strPadChC(const S: String; C: Char; Len: Integer): String;
function  strPadL(const S: String; Len: Integer): String;
function  strPadR(const S: String; Len: Integer): String;
function  strPadC(const S: String; Len: Integer): String;
function  strPadZeroL(const S: String; Len: Integer): String;
function  strRight(const S: String; Len: Integer): String;
function  strAddSlash(const S: String): String;
function  strDelSlash(const S: String): String;
function  strSpace(Len: Integer): String;
function  strTokenCount(S: String; Seperator: Char): Integer;
function  strTokenAt(const S:String; Seperator: Char; At: Integer): String;
procedure strTokenToStrings(S: String; Seperator: Char; List: TStrings);
function  strTokenFromStrings(Seperator: Char; List: TStrings): String;

function  strUpper(const S: String): String;
function  strEqual(const S1,S2: String): Boolean;
function  strComp(const S1,S2: String): Boolean;
function  strCompU(const S1,S2: String): Boolean;
function  strContains(const S1,S2: String): Boolean;
function  strContainsU(const S1,S2: String): Boolean;
{$ifdef SET}
function  strNiceNum(const S: String): String;
function  strNiceDateDefault(const S, Default: String): String;
function  strNiceDate(const S: String): String;
function  strNiceTime(const S: String): String;
function  strNicePhone(const S: String): String;
{$endif}
function  strReplace(const S: String; C: Char; const Replace: String): String;
function  strEncrypt(const S: String; Key: Word): String;
function  strDecrypt(const S: String; Key: Word): String;
function  strLastCh(const S: String): Char;
function  Group(Ch: Char): Char;
function  strSoundex(S: String): String;
function  strProfile(const aFile, aSection, aEntry, aDefault: String): String;
function  strCapitalize(const S: String): String;  { 31.07.96 sb }

{$ifdef SET}
procedure strSearchReplace(var S:String; const Source, Dest: String; Options: TSRoptions);
{$endif}
procedure strChange(var S:String; const Source, Dest: String);
procedure strStripLast(var S: String);
function  strToken(var S: String; Seperator: Char): String;

{$IFDEF Win32a}
procedure strDebug(const S: String);
function  strFileLoad(const aFile: String): String;
procedure strFileSave(const aFile,aString: String);
{$ENDIF}

{ Integer functions }
function  intCenter(a,b: LongInt): LongInt;
function  intMax(a,b: LongInt): LongInt;
function  intMin(a,b: LongInt): LongInt;
function  intPow(Base,Expo: Integer): LongInt;
function  intPow10(Exponent: Integer): LongInt;
function  intSign(a: LongInt): Integer;
function  intZero(a: LongInt; Len: Integer): String;
function  intPrime(Value: Integer): Boolean;
function  intPercent(a, b: LongInt): LongInt;

{ Extendedingpoint functions }
function  fltAdd(P1,P2: Extended; Decimals: Integer): Extended;
function  fltDiv(P1,P2: Extended; Decimals: Integer): Extended;
function  fltEqual(P1,P2: Extended; Decimals: Integer): Boolean;
function  fltEqualZero(P: Extended): Boolean;
function  fltGreaterZero(P: Extended): Boolean;
function  fltLessZero(P: Extended): Boolean;
function  fltNeg(P: Extended; Negate: Boolean): Extended;
function  fltMul(P1,P2: Extended; Decimals: Integer): Extended;
function  fltRound(P: Extended; Decimals: Integer): Extended;
function  fltSub(P1,P2: Extended; Decimals: Integer): Extended;
function  fltUnEqualZero(P: Extended): Boolean;
function  fltCalc(const Expr: String): Extended;
function  fltPower(a,n: Extended): Extended;
function  fltPositiv(Value: Extended): Extended;
function  fltNegativ(Value: Extended): Extended;

{ Rectangle functions from Golden Software }
function  rectHeight(const R: TRect): Integer;
function  rectWidth(const R: TRect): Integer;
function  rectSet(Left, Top, Right, Bottom: Integer): TRect;
function  rectInclude(const R1, R2: TRect): Boolean;
function  rectPoint(const R: TRect; P: TPoint): Boolean;
function  rectSetPoint(const TopLeft, BottomRight: TPoint): TRect;
function  rectIntersection(const R1, R2: TRect): TRect;
function  rectIsIntersection(const R1, R2: TRect): Boolean;
function  rectIsValid(const R: TRect): Boolean;
function  rectNull: TRect;
function  rectIsNull(const R: TRect): Boolean;
function  rectIsSquare(const R: TRect): Boolean;
function  rectCentralPoint(const R: TRect): TPoint;
function  rectBounds(aLeft,aTop,aWidth,aHeight: Integer): TRect;
procedure rectGrow(var R: TRect; Delta: Integer);
procedure rectRelativeMove(var R: TRect; DX, DY: Integer);
procedure rectMoveTo(var R: TRect; X, Y: Integer);

{ date functions }
function  dateYear(D: TDateTime): Integer;
function  dateMonth(D: TDateTime): Integer;
function  dateDay(D: TDateTime): Integer;
function  dateBeginOfYear(D: TDateTime): TDateTime;
function  dateEndOfYear(D: TDateTime): TDateTime;
function  dateBeginOfMonth(D: TDateTime): TDateTime;
function  dateLeapYear(D: TDateTime): Boolean;
function  dateBeginOfQuarter(D: TDateTime): TDateTime;
function  dateDaysInMonth(D: TDateTime): Integer;
{function  dateDiff(D1,D2: TDateTime): Integer;}

{ time functions }
function  timeHour(T: TDateTime): Integer;
function  timeMin(T: TDateTime): Integer;
function  timeSec(T: TDateTime): Integer;

{$IFDEF Win32a}
function  timeZoneOffset: Integer;
{$ENDIF}

{ com Functions }
function  comIsCis(const S: String): Boolean;
function  comIsInt(const S: String): Boolean;
function  comCisToInt(const S: String): String;
function  comIntToCis(const S: String): String;
function  comFaxToCis(const S: String): String;
function  comNormFax(const Name,Fax: String): String;
{$ifdef SET}
function  strValueAt(const S:String; At: Integer): String;
function  comNormPhone(const Phone: String): String;
{$endif}
function  comNormInt(const Name,Int: String): String;
function  comNormCis(const Name,Cis: String): String;

{$ifdef FFUNC}
{ file functions }
procedure fileShredder(const Filename: String);
function  fileSize(const Filename: String): Longint;
function  fileWildcard(const Filename: String): Boolean;
function  fileShellOpen(const aFile: String): Boolean;
function  fileShellPrint(const aFile: String): Boolean;
{$endif}

{$IFDEF Win32a}
function  fileTemp(const aExt: String): String;
function  fileExec(const aCmdLine: String; aHide, aWait: Boolean): Boolean;
function  fileLongName(const aFile: String): String;
function  fileShortName(const aFile: String): String;
{$ENDIF}
function  ExtractName(const Filename: String): String;

{ system functions }

{$IFDEF Win32a}
procedure sysSaverRunning(Active: Boolean);
{$ENDIF}

{ registry functions }

{$IFDEF Win32a}
function  regReadString(aKey: hKey; const Path: String): String;
procedure regWriteString(aKey: hKey; const Path,Value: String);
procedure regDelValue(aKey: hKey; const Path: String);
function  regInfoString(const Value: String): String;
function  regCurrentUser: String;
function  regCurrentCompany: String;
procedure regWriteShellExt(const aExt,aCmd,aMenu,aExec: String);

{ The following five functions came from David W. Yutzy / Celeste Software Services
  Thanks for submitting us the methods !!
}
procedure regKeyList(aKey: HKEY; const Path:String; var aValue: TStringList);
function  regValueExist(aKey: HKEY; const Path:String):Boolean;
function  regWriteValue(aKey: HKEY; const Path: String; Value: Variant; Typ: TDataType): Boolean;
function  regReadValue(aKey:HKEY; const Path:String; Typ: TDataType): Variant;
procedure regValueList(aKey: HKEY; const Path:String; var aValue: TStringList);
{$ENDIF}

end;

implementation

uses
  SysUtils, Controls, Forms, Consts, Dialogs;

{ bit manipulating }
{$ifdef SET}
function XProc.bitSet(const Value: LongInt; const TheBit: TBit): Boolean;
begin
  Result:= (Value and (1 shl TheBit)) <> 0;
end;

function XProc.bitOn(const Value: LongInt; const TheBit: TBit): LongInt;
begin
  Result := Value or (1 shl TheBit);
end;

function XProc.bitOff(const Value: LongInt; const TheBit: TBit): LongInt;
begin
  Result := Value and ((1 shl TheBit) xor $FFFFFFFF);
end;

function XProc.bitToggle(const Value: LongInt; const TheBit: TBit): LongInt;
begin
  result := Value xor (1 shl TheBit);
end;
{$endif}

{ string methods }

function XProc.strHash(const S: String; LastBucket: Integer): Integer;
var
  i: Integer;
begin
  Result:=0;
  for i := 1 to Length(S) do
    Result := ((Result shl 3) xor Ord(S[i])) mod LastBucket;
end;

function XProc.strTrim(const S: String): String;
begin
  Result:=StrTrimChR(StrTrimChL(S,BLANK),BLANK);
end;

function XProc.strTrimA(const S: String): String;
begin
  Result:=StrTrimChA(S,BLANK);
end;

function XProc.strTrimChA(const S: String; C: Char): String;
var
  I               : Word;
begin
  Result:=S;
  for I:=Length(Result) downto 1 do
    if Result[I]=C then Delete(Result,I,1);
end;

function XProc.strTrimChL(const S: String; C: Char): String;
begin
  Result:=S;
  while (Length(Result)>0) and (Result[1]=C) do Delete(Result,1,1);
end;

function XProc.strTrimChR(const S: String; C: Char): String;
begin
  Result:=S;
  while (Length(Result)> 0) and (Result[Length(Result)]=C) do
    Delete(Result,Length(Result),1);
end;

function XProc.strLeft(const S: String; Len: Integer): String;
begin
  Result:=Copy(S,1,Len);
end;

function XProc.strLower(const S: String): String;
begin
  Result:=AnsiLowerCase(S);
end;

function XProc.strMake(C: Char; Len: Integer): String;
begin
  Result:=strPadChL('',C,Len);
end;

function XProc.strPadChL(const S: String; C: Char; Len: Integer): String;
begin
  Result:=S;
  while Length(Result)<Len do Result:=C+Result;
end;

function XProc.strPadChR(const S: String; C: Char; Len: Integer): String;
begin
  Result:=S;
  while Length(Result)<Len do Result:=Result+C;
end;

function XProc.strPadChC(const S: String; C: Char; Len: Integer): String;
begin
  Result:=S;
  while Length(Result)<Len do
  begin
    Result:=Result+C;
    if Length(Result)<Len then Result:=C+Result;
  end;
end;

function XProc.strPadL(const S: String; Len: Integer): String;
begin
  Result:=strPadChL(S,BLANK,Len);
end;

function XProc.strPadC(const S: String; Len: Integer): String;
begin
  Result:=strPadChC(S,BLANK,Len);
end;


function XProc.strPadR(const S: String; Len: Integer): String;
begin
  Result:=strPadChR(S,BLANK,Len);
end;

function XProc.strPadZeroL(const S: String; Len: Integer): String;
begin
  Result:=strPadChL(strTrim(S),ZERO,Len);
end;

function XProc.strCut(const S: String; Len: Integer): String;
begin
  Result:=strLeft(strPadR(S,Len),Len);
end;

function XProc.strRight(const S: String; Len: Integer): String;
begin
  if Len>=Length(S) then
    Result:=S
  else
    Result:=Copy(S,Succ(Length(S))-Len,Len);
end;

function XProc.strAddSlash(const S: String): String;
begin
  Result:=S;
  if strLastCh(Result)<>SLASH then Result:=Result+SLASH;
end;

function XProc.strDelSlash(const S: String): String;
begin
  Result:=S;
  if strLastCh(Result)=SLASH then Delete(Result,Length(Result),1);
end;

function XProc.strSpace(Len: Integer): String;
begin
  Result:=StrMake(BLANK,Len);
end;

function XProc.strToken(var S: String; Seperator: Char): String;
var
  I               : Word;
begin
  I:=Pos(Seperator,S);
  if I<>0 then
  begin
    Result:=System.Copy(S,1,I-1);
    System.Delete(S,1,I);
  end else
  begin
    Result:=S;
    S:='';
  end;
end;

function XProc.strTokenCount(S: String; Seperator: Char): Integer;
var
  Res: String;

begin
  Result:=0;
  Res := '.';
  while (Res <> '.') do
  begin
    Res := StrToken(S,Seperator);
    if (Res <> '') then
      inc(result);
  end;
end;

function XProc.strTokenAt(const S:String; Seperator: Char; At: Integer): String;
var
  j,i: Integer;
begin
  Result:='';
  j := 1;
  i := 0;
  while (i<=At ) and (j<=Length(S)) do
  begin
    if S[j]=Seperator then
       Inc(i)
    else if i = At then
       Result:=Result+S[j];
    Inc(j);
  end;
end;

procedure XProc.strTokenToStrings(S: String; Seperator: Char; List: TStrings);
var
  Token: String;
begin
  List.Clear;
  Token:= strToken(S,Seperator);
  while Token<>'' do
  begin
    List.Add(Token);
    Token:=strToken(S,Seperator);
  end;
end;

function XProc.strTokenFromStrings(Seperator: Char; List: TStrings): String;
var
  i: Integer;
begin
  Result:='';
  for i:=0 to List.Count-1 do
     if Result<>'' then
       Result:=Result+Seperator+List[i]
     else
       Result:=List[i];
end;

function XProc.strUpper(const S: String): String;
begin
  Result:=AnsiUpperCase(S);
end;

function XProc.strEqual(const S1,S2: String): Boolean;
begin
  Result:=AnsiCompareText(S1,S2)=0;
end;

function XProc.strCompU(const S1,S2: String) : Boolean;
begin
  Result:=strEqual(strLeft(S2,Length(S1)),S1);
end;

function XProc.strComp(const S1,S2: String) : Boolean;
begin
  Result:=strLeft(S2,Length(S1))=S1;
end;

function XProc.strContains(const S1,S2: String): Boolean;
begin
  Result:=Pos(S1,S2) > 0;
end;

function XProc.strContainsU(const S1,S2: String): Boolean;
begin
  Result:=strContains(strUpper(S1),strUpper(S2));
end;

{$ifdef SET}
function XProc.strNiceNum(const S: String) : String;
var
  i    : Integer;
  Seps : set of Char;
begin
  Seps:=[ThousandSeparator,DecimalSeparator];
  Result:= ZERO;
  for i := 1 to Length(S) do
    if S[i] in DIGITS + Seps then
    begin
      if S[i] = ThousandSeparator then
         Result:=Result+DecimalSeparator
      else
         Result:=Result+S[i];
      if S[i] In Seps then Seps:=[];
    end
end;

function XProc.strNiceDate(const S: String): String;
begin
  Result:=strNiceDateDefault(S, DateToStr(Date));
end;

function XProc. strNiceDateDefault(const S, Default: String): String;
(* sinn der Procedure:
   Irgendeinen String bergeben und in ein leidlich brauchbares Datum verwandeln.
   Im Wesentlichen zum Abfangen des Kommazeichens auf dem Zehnerfeld.
   eingabe 10 = Rckgabe 10 des Laufenden Monats
   eingabe 10.12 = Rckgabe des 10.12. des laufenden Jahres.
   eingabe 10.12.96 = Rckgabe des Strings
   eingabe 10,12,96 = Rckgabe 10.12.95 (wird dann won STRtoDATE() gefressen)
   Eine Plausbilittskontrolle des Datums findet nicht Statt.
   Geplante Erweiterung:
   eingabe: +14  = Rckgabe 14 Tage Weiter
   eingabe: +3m  = Rckgabe 3 Monate ab Heute
   eingabe: +3w  = Rckgabe 3 Wochen (3*7 Tage) ab Heute
   Das gleiche auch Rckwrts mit  Minuszeichen
   eingabe: e oder E oder f  = Nchster Erster
   eingabe: e+1m Erster des bernchsten Monats
   Da lt sich aber noch trefflich weiterspinnen

   EV. mit Quelle rausgeben, damit sich die Englnder und Franzosen an
   Ihren Datumsformaten selbst erfreuen knnen und wir die passenden umsetzungen
   bekommen. *)
var
  a        : array [0..2] of string[4];
  heute    : string;
  i,j      : integer;
begin
  a[0]:='';
  a[1]:='';
  a[2]:='';
  heute := Default;

  j := 0;
  for i := 0 to length(S) do
    if S[i] in DIGITS then
      a[j] := a[j]+S[i]
    else if S[i] in [DateSeparator] then Inc(j);
  for i := 0 to 2 do
  if Length(a[i]) = 0 then
    if I=2 then
      a[i] :=copy(heute,i*3+1,4)
    else
      a[i] := copy(heute,i*3+1,2)
  else
    if length(a[i]) = 1 then
      a[i] := '0'+a[i];

  Result:=a[0]+DateSeparator+a[1]+DateSeparator+a[2];
  try
    (*StrToDate(Result);*)
  except
    Result:=DateToStr(Date);
  end;
end;

function XProc.strNiceTime(const S: String): String;
var
  a   : array[0..2] of string[2];
  i,j : integer;
begin
  j:= 0;
  a[0]:= '';
  a[1]:='';
  a[2]:='';
  for i:= 1 to length(S) do
  begin
    if S[i] in DIGITS then
    begin
      a[j] := a[j]+S[i];
    end
    else if S[i] in ['.',',',':'] then
      inc(J);
    if j > 2 then exit;
  end;
  for J := 0 to 2 do
    if length(a[j]) = 1 then a[j] := '0'+a[j] else
    if length(a[j]) = 0 then a[j] := '00';
  Result := a[0]+TimeSeparator+a[1]+TimeSeparator+a[2];
end;

function XProc.strNicePhone(const S: String): String;
var
  L : Integer;
begin
  if Length(S) > 3 then
  begin
    L:=(Length(S)+1) div 2;
    Result:=strNicePhone(strLeft(S,L))+SPACE+strNicePhone(strRight(S,Length(S)-L));
  end else
    Result := S;
end;
{$endif}

function XProc.strReplace(const S: String; C: Char; const Replace: String): String;
var
  i : Integer;
begin
  Result:='';
  for i:=Length(S) downto 1 do
    if S[i]=C then Result:=Replace+Result
              else Result:=S[i]+Result;
end;

procedure XProc.strChange(var S:String; const Source, Dest: String);
var
  P : Integer;
begin
  P:=Pos(Source,S);
  while P<>0 do
  begin
    Delete(S,P,Length(Source));
    Insert(Dest,S,P);
    P:=Pos(Source,S);
  end;
end;

{ sends a string to debug windows inside the IDE }
{$IFDEF Win32a}
procedure XProc.strDebug(const S: String);
var
  P    : PChar;
  CPS  : TCopyDataStruct;
  aWnd : hWnd;
begin
  aWnd := FindWindow('TfrmDbgTerm', nil);
  if aWnd <> 0 then
  begin
    CPS.cbData := Length(S) + 2;
    GetMem(P, CPS.cbData);
    try
      StrPCopy(P, S+CR);
      CPS.lpData := P;
      SendMessage(aWnd, WM_COPYDATA, 0, LParam(@CPS));
    finally
      FreeMem(P, Length(S)+2);
    end;
  end;
end;
{$ENDIF}

const
  CvTable : array['B'..'Z'] of char = (
    '1', '2', '3', '0', '1',   {'B' .. 'F'}
    '2', '0', '0', '2', '2',   {'G' .. 'K'}
    '4', '5', '5', '0', '1',   {'L' .. 'P'}
    '2', '6', '2', '3', '0',   {'Q' .. 'U'}
    '1', '0', '2', '0', '2' ); {'V' .. 'Z'}

  function XProc.Group(Ch: Char): Char;
  begin
  {$ifdef SET}
    if (Ch in ['B' .. 'Z']) and not (Ch In ['E','H','I','O','U','W','Y']) then
       Result:=CvTable[Ch]
    else
       Result:='0';
  {$endif}
  end;

function XProc.strSoundex(S: String): String;

var
  i,j : Integer;
  aGroup,Ch  : Char;

begin
  Result := '000';
  if S='' then exit;

  S:= strUpper(S);
  i:= 2;
  j:= 1;
  while (i <= Length(S)) and ( j<=3) do
  begin
    Ch := S[i];
    aGroup := Group(Ch);
    if (aGroup <> '0') and (Ch <> S[i-1]) and
       ((J=1) or (aGroup <> Result[j-1])) and
       ((i>2) or (aGroup <> Group(S[1]))) then
    begin
      Result[j] :=aGroup;
      Inc(j);
    end;
    Inc(i);
  end; {while}

  Result:=S[1]+'-'+Result;
end;

const
  C1 = 52845;
  C2 = 22719;

function XProc.strEncrypt(const S: String; Key: Word): String;
var
  I: Integer;
begin
 {$IFDEF Win32}
  SetLength(Result,Length(S));
 {$ELSE}
   Result[0]:=Chr(Length(S));
 {$ENDIF}
  for I := 1 to Length(S) do begin
    Result[I] := Char(Ord(S[I]) xor (Key shr 8));
    Key := (Ord(Result[I]) + Key) * C1 + C2;
  end;
end;

function XProc.strDecrypt(const S: String; Key: Word): String;
var
  I: Integer;
begin
 {$IFDEF Win32}
  SetLength(Result,Length(S));
 {$ELSE}
   Result[0]:=Chr(Length(S));
 {$ENDIF}
  for I := 1 to Length(S) do begin
    Result[I] := char(Ord(S[I]) xor (Key shr 8));
    Key := (Ord(S[I]) + Key) * C1 + C2;
  end;
end;

function XProc. strLastCh(const S: String): Char;
begin
  Result:=S[Length(S)];
end;

procedure XProc.strStripLast(var S: String);
begin
  if Length(S) > 0 then Delete(S,Length(S),1);
end;

{$ifdef SET}
procedure XProc.strSearchReplace(var S:String; const Source, Dest: String; Options: TSRoptions);
var hs,hs1,hs2,hs3: String;
var i,j : integer;

begin
 if  srCase in Options then
  begin
   hs := s;
   hs3 := source;
  end
 else
  begin
   hs:= StrUpper(s);
   hs3 := StrUpper(Source);
  end;
 hs1:= '';
 I:= pos(hs3,hs);
 j := length(hs3);
 while i > 0 do
 begin
   delete(hs,1,i+j-1); {Anfang Rest gendert 8.7.96 KM}
   hs1 := Hs1+copy(s,1,i-1); {Kopieren gendert 8.7.96 KM}
   delete(s,1,i-1); {Lschen bis Anfang posgendert 8.7.96 KM}
   hs2 := copy(s,1,j); {Bis ende pos Sichern}
   delete(s,1,j); {Lschen bis ende Pos}
   if    (not (srWord in Options))
       or (pos(s[1],' .,:;-#''+*?=)(/&%$"!{[]}\~<>|') > 0) then
    begin
     {Quelle durch ziel erstzen}
     hs1 := hs1+dest;
    end
   else
    begin
     hs1 := hs1+hs2;
    end;
   if srall in options then
    I:= pos(hs3,hs)
   else
    i :=0;
  end;
  s:= hs1+s;
end;
{$endif}

function XProc. strProfile(const aFile, aSection, aEntry, aDefault: String): String;
var
  aTmp: array[0..255] of Char;
begin
 {$IFDEF Win32a}
   GetPrivateProfileString(PChar(aSection), PChar(aEntry),
      PChar(aDefault), aTmp, Sizeof(aTmp)-1, PChar(aFile));
   Result:=StrPas(aTmp);
 {$ENDIF}
end;

function XProc.strCapitalize(const S: String): String;  { 31.07.96 sb }
var
  i      : Integer;
  Ch     : Char;
  First  : Boolean;
begin
  First  := True;
  Result := '';
  for i:=1 to Length(S) do
  begin
    Ch:=S[i];
    if Ch in [SPACE,'-','.'] then
       First:=True
    else if First then
    begin
      Ch:=upcase(Ch);
      First:=False;
    end;
    Result:=Result+Ch;
  end;
end;

{$IFDEF Win32a}
function XProc.strFileLoad(const aFile: String): String;
var
  aStr : TStrings;
begin
  Result:='';
  aStr:=TStringList.Create;
  try
    aStr.LoadFromFile(aFile);
    Result:=aStr.Text;
  finally
    aStr.Free;
  end;
end;

procedure XProc.strFileSave(const aFile,aString: String);
var
  Stream: TStream;
begin
  Stream := TFileStream.Create(aFile, fmCreate);
  try
    Stream.WriteBuffer(Pointer(aString)^,Length(aString));
  finally
    Stream.Free;
  end;
end;
{$ENDIF}

{ Integer stuff }

function XProc.IntCenter(a,b: LongInt): LongInt;
begin
  Result:=a div 2 - b div 2;
end;

function XProc.IntMax(a,b: LongInt): LongInt;
begin
  if a>b then Result:=a else Result:=b;
end;

function XProc.IntMin(a,b: LongInt): LongInt;
begin
  if a<b then Result:=a else Result:=b;
end;

function XProc.IntPow(Base,Expo: Integer): LongInt;
var
  Loop             : Word;
begin
  Result:=1;
  for Loop:=1 to Expo do Result:=Result*Base;
end;

function XProc.IntPow10(Exponent: Integer): LongInt;
begin
  Result:=IntPow(10,Exponent);
end;

function XProc.IntSign(a: LongInt): Integer;
begin
  if a<0 then Result:=-1 else if a>0 then Result:=+1 else Result:= 0;
end;

function XProc.IntZero(a: LongInt; Len: Integer): String;
begin
  Result:=strPadZeroL(IntToStr(a),Len);
end;

function XProc.IntPrime(Value: Integer): Boolean;
var
  i : integer;
begin
  Result:=False;
  if Value mod 2 <> 0 then
  begin
    i := 1;
    repeat
      i := i + 2;
      Result:= Value mod i = 0
    until Result or ( i > Trunc(sqrt(Value)) );
    Result:= not Result;
  end;
end;

function XProc.IntPercent(a, b : LongInt): LongInt;
begin
  Result := Trunc((a / b)*100);
end;

{ Extendeding point stuff }

function XProc.FltAdd(P1,P2: Extended; Decimals: Integer): Extended;
begin
  P1    :=fltRound(P1,Decimals);
  P2    :=fltRound(P2,Decimals);
  Result:=fltRound(P1+P2,Decimals);
end;

function XProc.FltDiv(P1,P2: Extended; Decimals: Integer): Extended;
begin
  P1:=fltRound(P1,Decimals);
  P2:=fltRound(P2,Decimals);
  if P2=0.0 then P2:=FLTZERO;       { provide division by zero }
  Result:=fltRound(P1/P2,Decimals);
end;

function XProc.FltEqual(P1,P2: Extended; Decimals: Integer): Boolean;
var
  Diff            : Extended;
begin
  Diff:=fltSub(P1,P2,Decimals);
  Result:=fltEqualZero(Diff);
end;

function XProc.FltEqualZero(P: Extended): Boolean;
begin
  Result:=(P>-FLTZERO) and (P<FLTZERO);
end;

function XProc.FltGreaterZero(P: Extended): Boolean;
begin
  Result:=P>FLTZERO;
end;

function XProc.FltLessZero(P: Extended): Boolean;
begin
  Result:=P<-FLTZERO;
end;

function XProc.FltNeg(P: Extended; Negate: Boolean): Extended;
begin
  if Negate then Result:=-P else Result:=P;
end;

function XProc.FltMul(P1,P2: Extended; Decimals: Integer): Extended;
begin
  P1    :=fltRound(P1,Decimals);
  P2    :=fltRound(P2,Decimals);
  Result:=fltRound(P1*P2,Decimals);
end;

function XProc.FltRound(P: Extended; Decimals: Integer): Extended;
var
  Factor  : LongInt;
  Help    : Extended;
begin
  Factor:=IntPow10(Decimals);
  if P<0 then Help:=-0.5 else Help:=0.5;
  Result:=Int(P*Factor+Help)/Factor;
  if fltEqualZero(Result) then Result:=0.00;
end;

function XProc.FltSub(P1,P2: Extended; Decimals: Integer): Extended;
begin
  P1    :=fltRound(P1,Decimals);
  P2    :=fltRound(P2,Decimals);
  Result:=fltRound(P1-P2,Decimals);
end;

function XProc.FltUnEqualZero(P: Extended): Boolean;
begin
  Result:=(P<-FLTZERO) or (P>FLTZERO)
end;

function XProc.FltCalc(const Expr: String): Extended;
const
  STACKSIZE = 10;
var
  Stack   : array[0..STACKSIZE] of double;
  oStack  : array[0..STACKSIZE] of char;
  z,n     : double;
  i,j,m   : integer;
  Bracket : boolean;
begin
  Bracket:= False; j := 0; n:= 1;z:=0; m:=1;
  for i := 1 to Length(Expr) do
  begin
    if not Bracket  then
       case Expr[i] of
         '0' .. '9': begin
                       z:=z*10+ord(Expr[i])-ord('0');
                       n:=n*m;
                     end;
         ',',#46   : m := 10;
         '('       : Bracket := True; {hier Klammeranfang merken, Zhler!!}
         '*','x',
         'X',
         '/','+'   : begin
                       Stack[j] := z/n;
                       oStack[j] := Expr[i];
                       Inc(j);
                       m:=1;z:=0;n:=1;
                     end;
       end {case}
    else
       Bracket:= Expr[i]<> ')'; {hier Rekursiver Aufruf, Zhler !!};
  end;
  Stack[j] := z/n;
  for i := 1 to j do
    case oStack[i-1] of
      '*','x','X' :  Stack[i]:= Stack[i-1]*Stack[i];
      '/'         :  Stack[i]:= Stack[i-1]/Stack[i];
      '+'         :  Stack[i]:= Stack[i-1]+Stack[i];
    end;
  Result:= Stack[j];
end;

function XProc.fltPower(a, n: Extended): Extended;
begin
  Result:=Exp(n * Ln(a));
end;

function XProc.fltPositiv(Value: Extended): Extended;
begin
  Result:=Value;
  if Value < 0.0 then Result:= 0.0;
end;

function XProc.fltNegativ(Value: Extended): Extended;
begin
  Result:=Value;
  if Value > 0.0 then Result:= 0.0;
end;

{ Rectangle Calculations }

function XProc.RectHeight(const R: TRect): Integer;
begin
  Result := R.Bottom - R.Top;
end;

function XProc.RectWidth(const R: TRect): Integer;
begin
  Result := R.Right - R.Left;
end;

procedure XProc.RectGrow(var R: TRect; Delta: Integer);
begin
    Dec(R.Left, Delta);
    Dec(R.Top, Delta);
    Inc(R.Right, Delta);
    Inc(R.Bottom, Delta);
end;

procedure XProc.RectRelativeMove(var R: TRect; DX, DY: Integer);
begin
    Inc(R.Left, DX);
    Inc(R.Right, DX);
    Inc(R.Top, DY);
    Inc(R.Bottom, DY);
end;

procedure XProc.RectMoveTo(var R: TRect; X, Y: Integer);
begin
    R.Right := X + R.Right - R.Left;
    R.Bottom := Y + R.Bottom - R.Top;
    R.Left := X;
    R.Top := Y;
end;

function XProc.RectSet(Left, Top, Right, Bottom: Integer): TRect;
begin
  Result.Left := Left;
  Result.Top := Top;
  Result.Right := Right;
  Result.Bottom := Bottom;
end;

function XProc.RectSetPoint(const TopLeft, BottomRight: TPoint): TRect;
begin
  Result.TopLeft := TopLeft;
  Result.BottomRight := BottomRight;
end;

function XProc.RectInclude(const R1, R2: TRect): Boolean;
begin
  Result := (R1.Left >= R2.Left) and (R1.Top >= R2.Top)
    and (R1.Right <= R2.Right) and (R1.Bottom <= R2.Bottom);
end;

function XProc. RectPoint(const R: TRect; P: TPoint): Boolean;
begin
  Result := (p.x>r.left) and (p.x<r.right) and (p.y>r.top) and (p.y<r.bottom);
end;

function XProc.RectIntersection(const R1, R2: TRect): TRect;
begin
  Result.Left := intMax(R1.Left, R2.Left);
  Result.Top := intMax(R1.Top, R2.Top);
  Result.Right := intMin(R1.Right, R2.Right);
  Result.Bottom := intMin(R1.Bottom, R2.Bottom);

  if not RectIsValid(Result) then
    Result := RectSet(0, 0, 0, 0);
end;

function XProc.RectIsIntersection(const R1, R2: TRect): Boolean;
begin
  Result := not RectIsNull(RectIntersection(R1, R2));
end;

function XProc.RectIsValid(const R: TRect): Boolean;
begin
  Result := (R.Left <= R.Right) and (R.Top <= R.Bottom);
end;

function XProc.RectNull: TRect;
begin
  Result := RectSet(0, 0, 0, 0);
end;

function XProc.RectIsNull(const R: TRect): Boolean;
begin
  Result := (R.Left = 0) and (R.Right = 0) and (R.Top = 0) and (R.Bottom = 0);
end;

function XProc.RectIsSquare(const R: TRect): Boolean;
begin
  Result := RectHeight(R) = RectWidth(R);
end;

function XProc.RectCentralPoint(const R: TRect): TPoint;
begin
  Result.X := R.Left + (RectWidth(R) div 2);
  Result.Y := R.Top + (RectHeight(R) div 2);
end;

function XProc. rectBounds(aLeft,aTop,aWidth,aHeight: Integer): TRect;
begin
  Result:=rectSet(aLeft,aTop,aLeft+aWidth,aTop+aHeight);
end;

{$ifdef FFUNC}
{ file functions }

procedure XProc.fileShredder(const Filename: String);
var
  aFile : Integer;
  aSize : Integer;
  P     : Pointer;
begin
  aSize:=fileSize(Filename);
  aFile:=FileOpen(FileName,fmOpenReadWrite);
  try
    Getmem(P,aSize);
    fillchar(P^,aSize,'X');
    FileWrite(aFile,P^,aSize);
    Freemem(P,aSize);
  finally
    FileClose(aFile);
    DeleteFile(Filename);
  end;
end;

function XProc.fileSize(const FileName: String): LongInt;
var
  SearchRec       : TSearchRec;
begin                                       { !Win32! -> GetFileSize }
  if FindFirst(FileName,faAnyFile,SearchRec)=0
    then Result:=SearchRec.Size
    else Result:=0;
end;

function XProc.fileWildcard(const Filename: String): Boolean;
begin
  Result:=(Pos('*',Filename)<>0) or (Pos('?',Filename)<>0);
end;

function XProc.fileShellOpen(const aFile: String): Boolean;
var
  Tmp: array[0..100] of char;
begin
  Result := ShellExecute( Application.Handle,
    'open', StrPCopy(Tmp,aFile), nil, nil, SW_NORMAL) > 32;
end;

function XProc.fileShellPrint(const aFile: String): Boolean;
var
  Tmp: array[0..100] of char;
begin
  Result := ShellExecute( Application.Handle,
    'print', StrPCopy(Tmp,aFile), nil, nil, SW_HIDE) > 32;
end;
{$endif}

{$IFDEF Win32a}
function XProc.fileTemp(const aExt: String): String;
var
  Buffer: array[0..1023] of Char;
  aFile : String;
begin
  GetTempPath(Sizeof(Buffer)-1,Buffer);
  GetTempFileName(Buffer,'TMP',0,Buffer);
  SetString(aFile, Buffer, StrLen(Buffer));
  Result:=ChangeFileExt(aFile,aExt);
  RenameFile(aFile,Result);
end;

function XProc.fileExec(const aCmdLine: String; aHide, aWait: Boolean): Boolean;
begin
end;


function XProc. fileLongName(const aFile: String): String;
var
  aInfo: TSHFileInfo;
begin
  if SHGetFileInfo(PChar(aFile),0,aInfo,Sizeof(aInfo),SHGFI_DISPLAYNAME)<>0 then
     Result:=StrPas(aInfo.szDisplayName)
  else
     Result:=aFile;
end;

function XProc. fileShortName(const aFile: String): String;
var
  aTmp: array[0..255] of char;
begin
  if GetShortPathName(PChar(aFile),aTmp,Sizeof(aTmp)-1)=0 then
     Result:=aFile
  else
     Result:=StrPas(aTmp);
end;

{$ENDIF}

function XProc.ExtractName(const Filename: String): String;
var
  aExt : String;
  aPos : Integer;
begin
  aExt:=ExtractFileExt(Filename);
  Result:=ExtractFileName(Filename);
  if aExt <> '' then
  begin
    aPos:=Pos(aExt,Result);
    if aPos>0 then
       Delete(Result,aPos,Length(aExt));
  end;
end;

{ date calculations }

function XProc. dateYear(D: TDateTime): Integer;
var
  Year,Month,Day : Word;
begin
  DecodeDate(D,Year,Month,Day);
  Result:=Year;
end;

function XProc. dateMonth(D: TDateTime): Integer;
var
  Year,Month,Day : Word;
begin
  DecodeDate(D,Year,Month,Day);
  Result:=Month;
end;

function XProc. dateBeginOfYear(D: TDateTime): TDateTime;
var
  Year,Month,Day : Word;
begin
  DecodeDate(D,Year,Month,Day);
  Result:=EncodeDate(Year,1,1);
end;

function XProc. dateEndOfYear(D: TDateTime): TDateTime;
var
  Year,Month,Day : Word;
begin
  DecodeDate(D,Year,Month,Day);
  Result:=EncodeDate(Year,12,31);
end;

function XProc. dateBeginOfMonth(D: TDateTime): TDateTime;
var
  Year,Month,Day : Word;
begin
  DecodeDate(D,Year,Month,Day);
  Result:=EncodeDate(Year,Month,1);
end;

function XProc.dateLeapYear(D: TDateTime): Boolean;
var
  Year,Month,Day: Word;
begin
  DecodeDate(D,Year,Month,Day);
  Result:=(Year mod 4 = 0) and ((Year mod 100 <> 0) or (Year mod 400 = 0));
end;

function XProc.dateBeginOfQuarter(D: TDateTime):TDateTime;
var
  Year,Month,Day : Word;
begin
  DecodeDate(D,Year,Month,Day);
  Result:=EncodeDate(Year,((Month-1 div 3) * 3)+1,1);
end;


function XProc.dateDaysInMonth(D: TDateTime): Integer;
const
  DaysPerMonth: array[1..12] of Byte= (31,28,31,30,31,30,31,31,30,31,30,31);
var
  Month: Integer;
begin
  Month:=dateMonth(D);
  Result:=DaysPerMonth[Month];
  if (Month=2) and dateLeapYear(D) then Inc(Result);
end;

function XProc.dateDay(D: TDateTime): Integer;
var
  Year,Month,Day : Word;
begin
  DecodeDate(D,Year,Month,Day);
  Result:=Day;
end;

{ time functions }

function XProc. timeHour(T: TDateTime): Integer;
var
  Hour,Minute,Sec,Sec100: Word;
begin
  DecodeTime(T,Hour,Minute,Sec,Sec100);
  Result:=Hour;
end;

function XProc. timeMin(T: TDateTime): Integer;
var
  Hour,Minute,Sec,Sec100: Word;
begin
  DecodeTime(T,Hour,Minute,Sec,Sec100);
  Result:=Minute;
end;

function XProc. timeSec(T: TDateTime): Integer;
var
  Hour,Minute,Sec,Sec100: Word;
begin
  DecodeTime(T,Hour,Minute,Sec,Sec100);
  Result:=Sec;
end;

{$IFDEF Win32a}
function XProc. timeZoneOffset: Integer;
var
  aTimeZoneInfo : TTimeZoneInformation;
begin
  if GetTimeZoneInformation(aTimeZoneInfo)<>-1 then
     Result := aTimeZoneInfo.Bias
  else
     Result := 0;
end;
{$ENDIF}

{ Communications Functions }

function XProc. comIsCis(const S: String): Boolean;
var
  T,
  aSt: String;
  PreId,
  PostId: Integer;
begin
  Result:=strContainsU('@compuserve.com',S);     { 28.7.96 sb This is also on CIS }
  if not Result then
     if Pos(',',S) > 0 then
     try
       aSt:=S;
       T := strToken(aSt,',');
       PreId:=StrToInt(T);
       PostId:=StrToInt(aSt);
       Result:=(PreId > 0) and (PostId > 0);
     except
       Result:=False;
     end;
end;

function XProc. comIsInt(const S: String): Boolean;
var
  aSt : String;
  PreId,
  PostId : String;
begin
  try
    aSt:=S;
    PreId:=strToken(aSt,'@');
    PostId:=aSt;
    Result:=(Length(PreId)>0) and (Length(PostId)>0);
  except
    Result:=False;
  end;
end;

{ converts a CIS adress to a correct Internet adress }
function XProc. comCisToInt(const S: String): String;
var
  P : Integer;
begin
  p:=Pos('INTERNET:',S);
  if P=1 then
    Result:=Copy(S,P+1,Length(S))
  else
  begin
    Result:=S;
    P:=Pos(',',Result);
    if P>0 then Result[P]:='.';
    Result:=Result+'@compuserve.com';     { 22.07.96 sb  Error }
  end;
end;

{ converts a internet adress to a correct CServe adress }
function XProc. comIntToCis(const S: String): String;
var
  P : Integer;
begin
  p:=Pos('@COMPUSERVE.COM',strUpper(S));
  if p > 0 then
  begin
    Result:=strLeft(S,P-1);
    P:=Pos('.',Result);
    if P>0 then Result[P]:=',';
  end else
    Result:='INTERNET:'+S;
end;

{ converts a fax adress to a correct CServe adress }
function XProc. comFaxToCis(const S: String): String;
begin
  Result:='FAX:'+S;
end;

function XProc.comNormFax(const Name, Fax: String): String;
begin
  if Name<>'' then
     Result:=Name+'[fax: '+Name+'@'+strTrim(Fax)+']'
  else
     Result:='[fax: '+strTrim(Fax)+']';
end;

function XProc. comNormInt(const Name,Int: String): String;
begin
  Result:='';
  if comIsInt(Int) then
     if Name <> '' then
        Result := Name + '|smtp: ' + strTrim(Int)
     else
        Result := 'smtp: ' + strTrim(Int);
end;

function XProc. comNormCis(const Name,Cis: String): String;
begin
  Result:='';
  if Name <> '' then
     Result := Name + '[compuserve: ' + strTrim(Cis) + ']'
  else
     Result := '[compuserve: ' + strTrim(Cis) + ']';
end;

{$ifdef SET}
  function XProc.strValueAt(const S:String; At: Integer): String;
  const
    Seperator = ',';
    Str = '"';
  var
    j,i: Integer;
    FSkip : Boolean;
  begin
    Result:='';
    j := 1;
    i := 0;
    FSkip:= False;
    while (i<=At ) and (j<=Length(S)) do
    begin
      if (S[j]=Str) then
         FSkip:=not FSkip
      else if (S[j]=Seperator) and not FSkip then
         Inc(i)
      else if i = At then
         Result:=Result+S[j];
      Inc(j);
    end;
  end;

function XProc. comNormPhone(const Phone: String): String;



var
  T,
  aNumber,
  aCountry,
  aPrefix,
  aDefault,
  aLocation  : String;

  i          : Integer;
begin
  aDefault  := '1,"Hamburg","","","40",49,0,0,0,"",1," "';
  aLocation := strProfile('telephon.ini','Locations','CurrentLocation','');
  if aLocation <> '' then
  begin
    aLocation:=strTokenAt(aLocation,',',0);
    if aLocation <> '' then
    begin
      aLocation:=strProfile('telephon.ini','Locations','Location'+aLocation,'');
      if aLocation <> '' then
         aDefault := aLocation;
    end;
  end;

  Result:='';
  aNumber:=strTrim(Phone);
  if aNumber <> '' then
    for i:=Length(aNumber) downto 1 do
      if not (aNumber[i] in DIGITS) then
      begin
        if aNumber[i] <> '+' then aNumber[i] := '-';
        if i < Length(aNumber) then                    { remove duplicate digits }
           if aNumber[i]=aNumber[i+1] then
              Delete(aNumber,i,1);
      end;

  if aNumber <> '' then
  begin
    if aNumber[1] = '+' then
       aCountry := strToken(aNumber,'-')
    else
       aCountry := '+'+strValueAt(aDefault,5);

    aNumber:=strTrimChL(aNumber,'-');

    if aNumber <> '' then
    begin
      if strTokenCount(aNumber,'-') > 1 then
      begin
         T := strToken(aNumber,'-');
         aPrefix := strTrimChL(T,'0');
      end
      else
         aPrefix := strValueAt(aDefault,4);

      aNumber:= strNicePhone(strTrimChA(aNumber,'-'));
      Result := aCountry + ' ('+aPrefix+') '+aNumber;
    end;
  end;
end;
{$endif}

{ system functions }

{$IFDEF Win32a}
procedure XProc.sysSaverRunning(Active: Boolean);
var
  aParam: Longint;
begin
  SystemParametersInfo (SPI_SCREENSAVERRUNNING, Word(Active),@aParam,0);
end;
{$ENDIF}

{ registry functions }

{$IFDEF Win32a }

procedure XProc.regParsePath(const Path: String; var aPath, aValue: String);
begin
  aPath:=Path;
  aValue:= '';
  while (Length(aPath)>0) and (strLastCh(aPath)<>'\') do
  begin
    aValue:=strLastCh(aPath)+aValue;
    strStripLast(aPath);
  end;
end;

function XProc.regReadString(aKey: HKEY; const Path: String): String;
begin
end;

procedure XProc.regWriteString(aKey: HKEY; const Path,Value: String);
begin
end;

procedure XProc.regDelValue(aKey: hKey; const Path: String);
begin
end;

function XProc.regInfoString(const Value: String): String;
begin
end;

function XProc.regCurrentUser: String;
begin
end;

function XProc.regCurrentCompany: String;
begin
end;

{ Add a shell extension to the registry }
procedure XProc.regWriteShellExt(const aExt,aCmd,aMenu,aExec: String);
var
  s, aPath : String;
begin
  with TRegistry.Create do
  try
    RootKey := HKEY_CLASSES_ROOT;
    aPath   := aExt;
    if KeyExists(aPath) then
    begin
      OpenKey(aPath,False);
      S:=ReadString('');
      CloseKey;
      if S<>'' then
         if KeyExists(S) then
            aPath:=S;
    end;

    OpenKey(aPath+'\Shell\'+aCmd,True);
    WriteString('',aMenu);
    CloseKey;

    OpenKey(aPath+'\Shell\'+aCmd+'\Command',True);
    WriteString('',aExec + ' %1');
    CloseKey;
  finally
    Free;
  end;
end;

procedure XProc.regValueList(aKey: HKEY; const Path:String; var aValue: TStringList);
var
  aRegistry: TRegistry;
begin
  aRegistry:=TRegistry.Create;
  try
    with aRegistry do
    begin
      RootKey:=aKey;
      OpenKey(Path,True);
      GetValueNames(aValue);
    end;
  finally
    aRegistry.Free;
  end;
end;

procedure XProc.regKeyList(aKey: HKEY; const Path:String; var aValue: TStringList);
var
  aRegistry: TRegistry;
begin
  aRegistry:=TRegistry.Create;
  try
    with aRegistry do
    begin
      RootKey:=aKey;
      OpenKey(Path,True);
      GetKeyNames(aValue);
    end;
  finally
    aRegistry.Free;
  end;
end;

function XProc.regValueExist(aKey: HKEY; const Path:String):Boolean;
var
  aRegistry: TRegistry;
  aPath: String;
  aValue: String;
begin
  aRegistry:=TRegistry.Create;
  try
    with aRegistry do
    begin
      RootKey:=aKey;
      regParsePath(Path, aPath, aValue);
      OpenKey(aPath,True);
      Result := ValueExists(aValue)
    end;
  finally
    aRegistry.Free;
  end;
end;

function XProc. regReadValue(aKey:HKEY; const Path:String; Typ: TDataType): Variant;
var
  aRegistry: TRegistry;
  aPath: String;
  aValue: String;
begin
  aRegistry:=TRegistry.Create;
  try
    with aRegistry do
    begin
      RootKey:=aKey;
      regParsePath(Path, aPath, aValue);
      if OpenKey(aPath,True) then
        if ValueExists(aValue) then
           case Typ of
             dtInteger:  Result := ReadInteger(aValue);
             dtBoolean:  Result := ReadBool(aValue);
             dtString:   Result := ReadString(aValue);
             dtDate:     Result := ReadDate(aValue);
             dtExtended:    Result := ReadExtended(aValue);
             dtCurrency: Result := ReadCurrency(aValue);
             dtTime:     Result := REadTime(aValue);
           end;
    end;
  finally
    aRegistry.Free;
  end;
end;

function XProc. regWriteValue(aKey: HKEY; const Path: String; Value: Variant; Typ: TDataType): Boolean;
var
  aRegistry : TRegistry;
  aPath     : String;
  aValue    : String;
begin
  aRegistry:=TRegistry.Create;
  try
    with aRegistry do
    begin
      RootKey:=aKey;
      regParsePath(Path, aPath, aValue);
      if OpenKey(aPath,True) then
        case Typ of
          dtInteger:  WriteInteger(aValue, Value);
          dtBoolean:  WriteBool(aValue, Value);
          dtString:   WriteString(aValue, Value);
          dtDate:     WriteDate(aValue, Value);
          dtExtended:    WriteExtended(aValue, Value);
          dtCurrency: WriteCurrency(aValue, Value);
          dtTime:     WriteTime(aValue, Value);
        end
      else
        Result := False;
    end;
  finally
    aRegistry.Free;
  end;
end;

{$ENDIF}

{ other stuff }

function MsgBox(const aTitle,aMsg: String; aFlag: Integer): Integer;
var
  ActiveWindow : hWnd;
  WindowList   : Pointer;
  TmpA         : array[0..200] of char;
  TmpB         : array[0..100] of char;
begin
  ActiveWindow:=GetActiveWindow;
  WindowList:= DisableTaskWindows(0);
  try
    StrPCopy(TmpB,aTitle);
    StrPCopy(TmpA,aMsg);
   {$IFDEF Win32}
    Result:=Windows.MessageBox(Application.Handle, TmpA, TmpB, aFlag);
   {$ELSE}
    Result:=WinProcs.MessageBox(Application.Handle, TmpA, TmpB, aFlag);
   {$ENDIF}
  finally
    EnableTaskWindows(WindowList);
    SetActiveWindow(ActiveWindow);
  end;
end;

initialization
  Randomize;
end.
